@@ -1,27 +1,27 @@
 /*
-getarsym.c - implementation of the elf_getarsym(3) function.
-Copyright (C) 1995 - 1998 Michael Riepe <michael@stud.uni-hannover.de>
-
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Library General Public
-License as published by the Free Software Foundation; either
-version 2 of the License, or (at your option) any later version.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Library General Public License for more details.
-
-You should have received a copy of the GNU Library General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/
+ * getarsym.c - implementation of the elf_getarsym(3) function.
+ * Copyright (C) 1995 - 1998, 2004 Michael Riepe
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 #include <private.h>
 #include <byteswap.h>
 
 #ifndef lint
-static const char rcsid[] = "@(#) $Id: getarsym.c,v 1.3 1998/06/12 19:42:24 michael Exp $";
+static const char rcsid[] = "@(#) $Id: getarsym.c,v 1.8 2005/05/21 15:39:23 michael Exp $";
 #endif /* lint */
 
 Elf_Arsym*
@@ -72,7 +72,7 @@ elf_getarsym(Elf *elf, size_t *ptr) {
 		return NULL;
 	    }
 	    elf_assert(!*s);
-	    syms[i].as_hash = elf_hash(syms[i].as_name);
+	    syms[i].as_hash = elf_hash((unsigned char*)syms[i].as_name);
 	    syms[i].as_off = __load_u32M(elf->e_symtab + 4 * (i + 1));
 	}
 	syms[count].as_name = NULL;
